﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.ComplexEventProcessing.Adapters
Imports Microsoft.ComplexEventProcessing

Namespace Adapters
    Public Class RandomPayloadInputAdapterFactory
        Implements ITypedInputAdapterFactory(Of RandomPayloadConfig), ITypedDeclareAdvanceTimeProperties(Of RandomPayloadConfig)

        Public Function Create(Of TPayload)(ByVal configInfo As RandomPayloadConfig, ByVal eventShape As EventShape) As InputAdapterBase Implements ITypedInputAdapterFactory(Of RandomPayloadConfig).Create
            If eventShape <> eventShape.Point Then
                Throw New InvalidOperationException("Point only.")
            End If

            Return New RandomPayloadPointInputAdapter(configInfo)
        End Function

        Public Sub Dispose() Implements ITypedInputAdapterFactory(Of RandomPayloadConfig).Dispose

        End Sub

        ''' <summary>
        ''' Specifies the CTI behavior of this adapter. This method will be called whenever
        ''' a new adapter instance is needed, with the respective configuration and event type.
        ''' </summary>
        ''' <typeparam name="TPayload">Type of the payload.</typeparam>
        ''' <param name="configInfo">Configuration passed from the query binding.</param>
        ''' <param name="eventShape">Event shape requested from the query binding.</param>
        ''' <returns>An instance of AdapterAdvanceTimeSettings.</returns>
        Public Function DeclareAdvanceTimeProperties(Of TPayload)(ByVal configInfo As RandomPayloadConfig, ByVal eventShape As EventShape) As AdapterAdvanceTimeSettings Implements ITypedDeclareAdvanceTimeProperties(Of RandomPayloadConfig).DeclareAdvanceTimeProperties
            ' Inject a CTI one tick _after_ (negative delay) each event and
            ' drop violating events. This ensures maximum liveliness, since
            ' each event is immediately committed.  Note that this will drop
            ' duplicate events (events collocated in time) when enqueued by
            ' the input adapter.
            Return New AdapterAdvanceTimeSettings(New AdvanceTimeGenerationSettings(configInfo.CtiFrequency, TimeSpan.FromTicks(-1)), AdvanceTimePolicy.Drop)
        End Function
    End Class
End Namespace