﻿Imports System
Imports System.Collections.Generic
Imports System.Linq
Imports System.Text
Imports Microsoft.ComplexEventProcessing
Imports DemoConsole.Adapters
Imports Microsoft.ComplexEventProcessing.Linq

Imports System.Data.Linq
Imports System.Xml.Linq

Friend Class Program
    Shared Sub Main(ByVal args() As String)
        Using _server = Server.Create("Default")
            'for shared server scenarios
            'Server.Connect(new EndpointAddress(@"http://localhost/StreamInsight/Default")))
            'for shared server scenarios
            If _server.Applications.ContainsKey("Msdn") Then
                _server.Applications("Msdn").Delete()
            End If
            Dim app = _server.CreateApplication("Msdn")

            Dim inputConfig = New RandomPayloadConfig With {.SleepTime = 200}

            Dim inputStream = CepStream(Of EventPayload).Create("input", GetType(RandomPayloadInputAdapterFactory), inputConfig, EventShape.Point)

            'Simple filtering query
            Dim filtered = From i In inputStream
                           Where i.Value > 0.5
                           Select i

            'Simple aggregation over time
            Dim aggregated = From i In inputStream.TumblingWindow(TimeSpan.FromSeconds(5), HoppingWindowOutputPolicy.ClipToWindowEnd)
                             Select New With {Key .Avg = i.Avg(Function(p) p.Value)}

            'Simple grouping query
            Dim payloadByRegionT = inputStream.GroupBy(Function(source) source.Region)

            Dim payloadByRegion = From c In payloadByRegionT.ApplyWithUnion(
                                  Function(item) item).HoppingWindow(
                                  TimeSpan.FromMinutes(1), TimeSpan.FromSeconds(2), HoppingWindowOutputPolicy.ClipToWindowEnd)
                Select New With {.Region = payloadByRegionT.Key, .Sum = c.Sum(Function(p) p.Value)}

            'TopK
            Dim highestRegion = (
                From i In payloadByRegion.SnapshotWindow(SnapshotWindowOutputPolicy.Clip)
                From sumByRegion In i
                Order By sumByRegion.Sum Descending
                Select sumByRegion).Take(1) 'uses groupBy query above

            'Simple count window
            'Count window only works with user defined aggregates                
            Dim sumEvery5 = From i In inputStream.CountByStartTimeWindow(5, CountWindowOutputPolicy.PointAlignToWindowEnd)
                    Select New With {Key .Sum = i.EveryOtherSum(Function(p) p.Value)}

            'Select query to run
            Dim template = filtered
            'var template = aggregated;
            'var template = payloadByRegion;                
            'var template = highestRegion;
            'var template = sumEvery5;

            Dim query = template.ToQuery(app, "query", "description", GetType(ConsoleOutputAdapterFactory), New ConsoleOutputAdapterConfig With {.OutputMessageFormatString = "Received event: {0}"}, EventShape.Point, StreamEventOrder.FullyOrdered)

            Console.WriteLine("Starting...")
            query.Start()

            Console.WriteLine("Started. Press Enter to Stop.")
            Console.ReadLine()

            query.Stop()
            Console.WriteLine("Stopped. Press Enter to quit.")
            Console.ReadLine()
        End Using
    End Sub
End Class

Public Class MyEnumerator
    Public Region As String
    Public Address As String
End Class